@@ -10,6 +10,21 @@
 # define _XOPEN_SOURCE_EXTENDED 1
 #endif
 
+/* Uncomment the following #define to test the 'classic' (undocumented
+SysV) mouse functions in inputTest.  Otherwise,  the ncurses mouse
+interface will be used.       */
+
+ #define CLASSIC_MOUSE_INTERFACE
+
+#if defined( CLASSIC_MOUSE_INTERFACE)
+   #define BUTTON_MOVE_EVENTS (BUTTON1_MOVED | BUTTON2_MOVED | BUTTON3_MOVED \
+                          | BUTTON4_MOVED | BUTTON5_MOVED)
+   #define ALL_MOVE_EVENTS  (BUTTON_MOVE_EVENTS | REPORT_MOUSE_POSITION)
+#else
+   #define PDC_NCMOUSE
+   #define ALL_MOVE_EVENTS   REPORT_MOUSE_POSITION
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <ctype.h>
@@ -108,18 +123,95 @@ COMMAND command[MAX_OPTIONS] =
 };
 
 int width, height;
-static bool report_mouse_movement = FALSE;
+static mmask_t test_mouse_mask = ALL_MOUSE_EVENTS;
 static SCREEN *screen_pointer;
 
+static mmask_t parse_mouse_mask( const char *str)
+{
+   test_mouse_mask = 0;
+   while( *str)
+      {
+      const mmask_t *event_ptr = NULL;
+      const mmask_t pressed[5] = { BUTTON1_PRESSED, BUTTON2_PRESSED,
+                  BUTTON3_PRESSED, BUTTON4_PRESSED, BUTTON5_PRESSED };
+      const mmask_t released[5] = { BUTTON1_RELEASED, BUTTON2_RELEASED,
+                  BUTTON3_RELEASED, BUTTON4_RELEASED, BUTTON5_RELEASED };
+      const mmask_t clicked[5] = { BUTTON1_CLICKED, BUTTON2_CLICKED,
+                  BUTTON3_CLICKED, BUTTON4_CLICKED, BUTTON5_CLICKED };
+      const mmask_t dblclk[5] = { BUTTON1_DOUBLE_CLICKED, BUTTON2_DOUBLE_CLICKED,
+                  BUTTON3_DOUBLE_CLICKED, BUTTON4_DOUBLE_CLICKED, BUTTON5_DOUBLE_CLICKED };
+      const mmask_t triclk[5] = { BUTTON1_TRIPLE_CLICKED, BUTTON2_TRIPLE_CLICKED,
+                  BUTTON3_TRIPLE_CLICKED, BUTTON4_TRIPLE_CLICKED, BUTTON5_TRIPLE_CLICKED };
+#ifdef BUTTON1_MOVED
+      const mmask_t moved[5] = { BUTTON1_MOVED, BUTTON2_MOVED,
+                  BUTTON3_MOVED, BUTTON4_MOVED, BUTTON5_MOVED };
+#endif
+
+      switch( *str)
+         {
+         case 'p':
+            event_ptr = pressed;
+            break;
+         case 'r':
+            event_ptr = released;
+            break;
+         case 'c':
+            event_ptr = clicked;
+            break;
+         case 'd':
+            event_ptr = dblclk;
+            break;
+         case 't':
+            event_ptr = triclk;
+            break;
+#ifdef BUTTON1_MOVED
+         case 'm':
+            event_ptr = moved;
+            break;
+#endif
+#ifdef MOUSE_WHEEL_SCROLL
+         case 'w':
+            test_mouse_mask |= MOUSE_WHEEL_SCROLL;
+            break;
+#endif
+#ifdef BUTTON_MODIFIER_SHIFT
+         case 's':
+            test_mouse_mask |= BUTTON_MODIFIER_SHIFT;
+            break;
+#endif
+#ifdef BUTTON_MODIFIER_CONTROL
+         case 'x':
+            test_mouse_mask |= BUTTON_MODIFIER_CONTROL;
+            break;
+#endif
+#ifdef BUTTON_MODIFIER_ALT
+         case 'a':
+            test_mouse_mask |= BUTTON_MODIFIER_ALT;
+            break;
+#endif
+         case 'M':
+            test_mouse_mask |= REPORT_MOUSE_POSITION;
+            break;
+         case 'W':
+            mouseinterval( atoi( str + 1));
+            break;
+         }
+      str++;
+      if( event_ptr)
+         while( *str >= '1' && *str <= '5')
+            test_mouse_mask |= event_ptr[*str++ - '1'];
+      }
+   return test_mouse_mask;
+}
+
 int main(int argc, char *argv[])
 {
     WINDOW *win;
     int key, old_option = -1, new_option = 0, i;
     bool quit = FALSE;
 
-#ifdef _WIN32
-    setlocale(LC_ALL, ".utf8");
-#endif
+    if( !setlocale( LC_CTYPE, "C.UTF-8"))
+        setlocale( LC_CTYPE, "en_US.utf8");
 
 #ifdef __PDCURSESMOD__
 #ifdef PDC_VER_MAJOR   /* so far only seen in 4.0+ */
@@ -135,7 +227,7 @@ int main(int argc, char *argv[])
             switch( argv[i][1])
             {
                 case 'l': case 'L':
-                    setlocale( LC_ALL, argv[i] + 2);
+                    setlocale( LC_CTYPE, argv[i] + 2);
                     break;
 #ifdef __PDCURSESMOD__
                 case 'b': case 'B':
@@ -162,7 +254,10 @@ int main(int argc, char *argv[])
 #endif
 #endif
                 case 'z':
-                    report_mouse_movement = TRUE;
+                    if( !argv[i][2])
+                       test_mouse_mask = ALL_MOUSE_EVENTS | REPORT_MOUSE_POSITION;
+                    else
+                       test_mouse_mask = parse_mouse_mask( argv[i] + 2);
                     break;
                 default:
                     break;
@@ -385,6 +480,9 @@ void inputTest(WINDOW *win)
     static const char spinner[5] = "/-\\|";
     int spinner_count = 0;
     int line = 3;
+#ifndef CLASSIC_MOUSE_INTERFACE
+    int mouse_buttons_held = 0;
+#endif
 
     wclear(win);
 
@@ -425,9 +523,14 @@ void inputTest(WINDOW *win)
 
     wtimeout(win, 200);
 
-#ifdef PDCURSES
-    mouse_set( ALL_MOUSE_EVENTS |
-            (report_mouse_movement ? REPORT_MOUSE_POSITION : 0));
+#ifdef CLASSIC_MOUSE_INTERFACE
+    mouse_set( test_mouse_mask);
+#else
+    mousemask( test_mouse_mask, NULL);
+#endif
+#ifdef NCURSES_VERSION
+    if( test_mouse_mask & REPORT_MOUSE_POSITION)
+      printf( "\x1b\x5b?1003h\n");        /* command string to enable mouse movement */
 #endif
     curs_set(0);        /* turn cursor off */
 
@@ -472,18 +575,15 @@ void inputTest(WINDOW *win)
             waddch( win, c);
         else
             wprintw(win, "%s", unctrl(c));
-#ifdef PDCURSES
+#ifdef CLASSIC_MOUSE_INTERFACE
         if (c == KEY_MOUSE)
         {
             int button = 0;
             request_mouse_pos();
 
-            if (BUTTON_CHANGED(1))
-                button = 1;
-            else if (BUTTON_CHANGED(2))
-                button = 2;
-            else if (BUTTON_CHANGED(3))
-                button = 3;
+            for( i = 0; i < PDC_MAX_MOUSE_BUTTONS; i++)
+                if (BUTTON_CHANGED(i))
+                    button = i;
 
             wmove(win, line, 5);
             wclrtoeol(win);
@@ -491,11 +591,7 @@ void inputTest(WINDOW *win)
                 wprintw(win, "Button %d: ", button);
 
             if (MOUSE_MOVED)
-            {
                 waddstr(win, "moved: ");
-                if( !button)
-                    button = 1;     /* to allow button modifiers to be read */
-            }
             else if (MOUSE_WHEEL_UP)
                 waddstr(win, "wheel up: ");
             else if (MOUSE_WHEEL_DOWN)
@@ -519,6 +615,8 @@ void inputTest(WINDOW *win)
             else
                 waddstr(win, "released: ");
 
+            if( !button)
+                button = 1;     /* to allow button modifiers to be read */
             if( BUTTON_STATUS(button) & BUTTON_MODIFIER_MASK)
             {
                 if (BUTTON_STATUS(button) & BUTTON_SHIFT)
@@ -548,6 +646,59 @@ void inputTest(WINDOW *win)
             if (PDC_get_key_modifiers() & PDC_KEY_MODIFIER_NUMLOCK)
                 waddstr(win, " NUMLOCK");
         }
+#else             /* ncurses mouse handling */
+        if (c == KEY_MOUSE)
+        {
+            MEVENT mevent;
+
+            wmove(win, line, 5);
+            wclrtoeol(win);
+            if( OK == getmouse( &mevent))
+            {
+                const mmask_t events[] = { BUTTON1_PRESSED, BUTTON1_RELEASED,
+                        BUTTON1_CLICKED, BUTTON1_DOUBLE_CLICKED, BUTTON1_TRIPLE_CLICKED,
+                        BUTTON2_PRESSED, BUTTON2_RELEASED, BUTTON2_CLICKED,
+                         BUTTON2_DOUBLE_CLICKED, BUTTON2_TRIPLE_CLICKED,
+                        BUTTON3_PRESSED, BUTTON3_RELEASED, BUTTON3_CLICKED,
+                         BUTTON3_DOUBLE_CLICKED, BUTTON3_TRIPLE_CLICKED,
+                        BUTTON4_PRESSED, BUTTON4_RELEASED, BUTTON4_CLICKED,
+                         BUTTON4_DOUBLE_CLICKED, BUTTON4_TRIPLE_CLICKED,
+                        BUTTON5_PRESSED, BUTTON5_RELEASED, BUTTON5_CLICKED,
+                         BUTTON5_DOUBLE_CLICKED, BUTTON5_TRIPLE_CLICKED,
+#ifdef BUTTON_CTRL
+                  BUTTON_CTRL,
+#else
+                  BUTTON_CONTROL,
+#endif
+                        BUTTON_SHIFT, BUTTON_ALT };
+
+                for( i = 0; (size_t)i < sizeof( events) / sizeof( events[0]); i++)
+                    if( mevent.bstate & events[i])
+                    {
+                        const char *text[8] = { "pressed", "released", "clicked",
+                               "dblclick", "triple-click", "Ctrl", "Shift", "Alt" };
+
+                        if( i < 25)
+                           wprintw( win, "button %d %s", i / 5 + 1, text[i % 5]);
+                        else
+                           wprintw( win, "  %s", text[i - 20]);
+                        if( i % 5 == 0)    /* button pressed */
+                           mouse_buttons_held |= (1 << (i / 5));
+                        if( i % 5 == 1)    /* button released */
+                           mouse_buttons_held &= ~(1 << (i / 5));
+                    }
+                if( mevent.bstate & ALL_MOVE_EVENTS)
+                {
+                    wprintw( win, "moved <%d>", mouse_buttons_held);
+                    for( i = 0; i < 3; i++)
+                        if( (mouse_buttons_held >> i) & 1)
+                            wprintw( win, "  Button %d", i + 1);
+                }
+                wprintw(win, "  Posn: Y: %d X: %d", mevent.y, mevent.x);
+            }
+            else
+                wprintw( win, "  ? getmouse failed ?");
+        }
 #endif
         wrefresh(win);
         line++;
@@ -559,9 +710,15 @@ void inputTest(WINDOW *win)
     wtimeout(win, -1);  /* turn off timeout() */
     curs_set(1);        /* turn cursor back on */
 
-#ifdef PDCURSES
+#ifdef CLASSIC_MOUSE_INTERFACE
     mouse_set(0L);
     PDC_return_key_modifiers(FALSE);
+#else
+    mousemask( (mmask_t)0, NULL);
+#endif
+#ifdef NCURSES_VERSION
+    if( test_mouse_mask & REPORT_MOUSE_POSITION)
+      printf( "\x1b\x5b?1003l\n");        /* disables mouse movement reports */
 #endif
     wclear(win);
     if( c == 1)
@@ -616,7 +773,7 @@ void inputTest(WINDOW *win)
     mvwaddstr(win, 6, 2, "Enter a number then a string separated by space");
     mvwin(win, 2, 1);
     wrefresh(win);
-    mvwscanw(win, 7, 6, "%d %s", &num, buffer);
+    mvwscanw(win, 7, 6, "%d %79s", &num, buffer);
     mvwprintw(win, 8, 6, "String: %s Number: %d", buffer, num);
     Continue(win);
 
@@ -746,11 +903,11 @@ void outputTest(WINDOW *win)
     mvwaddstr(win, 10, 1, "Enter a string: ");
     wrefresh(win);
     echo();
-    wscanw(win, "%s", Buffer);
+    wscanw(win, "%79s", Buffer);
 
     printw("This is a formatted string in stdscr: %d %s\n", 42, "is it");
     mvaddstr(10, 1, "Enter a string: ");
-    scanw("%s", Buffer);
+    scanw("%79s", Buffer);
 
     wclear(win);
     curs_set(2);
@@ -930,6 +1087,7 @@ void clipboardTest(WINDOW *win)
         clear();
         mvaddstr(1, 1, "Clipboard contents...");
         mvprintw(2, 1, "%s\n", ptr);
+        PDC_freeclipboard( ptr);
     }
 
     Continue2();
@@ -1471,6 +1629,8 @@ void extended(int tmarg)
     short i, x, y, z, lmarg = (short)(COLS - 77) / 2;
 
     erase();
+    if( lmarg < 0)
+      lmarg = 0;
 
     curs_set(0);
 
@@ -1481,9 +1641,17 @@ void extended(int tmarg)
     mvaddstr(tmarg + 2, lmarg, "6x6x6 Color Cube (16-231):");
 
     mvaddstr(tmarg + 4,  lmarg, "Blk      Red");
-    mvaddstr(tmarg + 4,  lmarg + 65, "Grn      Yel");
     mvaddstr(tmarg + 11, lmarg, "Blue    Mgta");
-    mvaddstr(tmarg + 11, lmarg + 65, "Cyan   White");
+    if( COLS >= 77)
+    {
+        mvaddstr(tmarg + 4,  lmarg + 65, "Grn      Yel");
+        mvaddstr(tmarg + 11, lmarg + 65, "Cyan   White");
+    }
+    else
+    {
+        mvaddstr(tmarg + 11, COLS - 12, "Grn      Yel");
+        mvaddstr(tmarg + 18, COLS - 12, "Cyan   White");
+    }
 
     for (i = 16; i < 256; i++)
         init_pair(i, COLOR_BLACK, i);
@@ -1493,8 +1661,14 @@ void extended(int tmarg)
             for (y = 0; y < 6; y++)
             {
                 chtype ch = ' ' | COLOR_PAIR(i++);
+                int line = tmarg + 5 + y, col = z * 13 + x * 2 + lmarg;
 
-                mvaddch(tmarg + 5 + y, z * 13 + x * 2 + lmarg, ch);
+                if( z == 5 && COLS < 77)
+                {
+                    line += 7;
+                    col += COLS - 77;
+                }
+                mvaddch( line, col, ch);
                 addch(ch);
             }
 
@@ -1627,6 +1801,8 @@ void gradient(int tmarg)
     curs_set(1);
 
     attrset(A_NORMAL);
+    if( cnum >= COLORS || pnum >= COLOR_PAIRS)
+       mvaddstr(tmarg + 18, 3, "RAN OUT OF COLORS");
     mvaddstr(tmarg + 19, 3, "Press any key to continue");
     curTest();
 }
@@ -1737,7 +1913,7 @@ void wideTest(WINDOW *win)
     {
         printw("%04x ", tmp[i]);
         addnwstr(tmp + i, 1);
-        addstr("  ");
+        addstr( getcurx( stdscr) > COLS - 8 ? "\n " : "  ");
     }
 
     noecho();
