@@ -1,4 +1,4 @@
-/* PDCurses */
+/* PDCursesMod */
 
 /* On Linux,  and probably some other platforms,  we can just
 use the built-in wcwidth() function.  */
@@ -142,11 +142,11 @@ PDCEX int PDC_wcwidth( const int32_t ucs)
 /*  A greatly modified version of Markus Kuhn's excellent
 wcwidth implementation.  For his latest version and many
 comments,  see http://www.cl.cam.ac.uk/~mgk25/ucs/wcwidth.c
-For PDCurses,  only PDC_wcwidth is used,  modified to take an
+For PDCursesMod,  only PDC_wcwidth is used,  modified to take an
 int argument instead of wchar_t,  because in MS-land, wchar_t
 is 16 bits;  getting the full Unicode range requires 21 bits.
 Also modified format/indenting to conform to PDCurses norms,
-and (June 2022) updated from Unicode 5.0 to 14.0.0.  See
+and (December 2024) updated from Unicode 5.0 to 16.0.0.  See
 uni_tbl.c in the Bill-Gray/junk repository.
 
 Following function modified from one in the README.md at
@@ -210,7 +210,7 @@ PDCEX int PDC_wcwidth( const int32_t ucs)
       /* see 'uni_tbl.c' in the Bill-Gray/junk repo */
 const uint16_t tbl_for_zero_width_chars[][2] = {
      { /* plane 0 */ 0, 202 },
-     { /* plane 1 */ 202, 312 },
+     { /* plane 1 */ 202, 333 },
      { /* plane 2 */ 0, 0 },
      { /* plane 3 */ 0, 0 },
      { /* plane 4 */ 0, 0 },
@@ -223,7 +223,7 @@ const uint16_t tbl_for_zero_width_chars[][2] = {
      { /* plane 11 */ 0, 0 },
      { /* plane 12 */ 0, 0 },
      { /* plane 13 */ 0, 0 },
-     { /* plane 14 */ 312, 313 },
+     { /* plane 14 */ 333, 334 },
      { /* plane 15 */ 0, 0 },
      { /* plane 16 */ 0, 0 },
      { 0x00AD, 0x036F }, { 0x0483, 0x0489 }, { 0x0591, 0x05BD },
@@ -254,7 +254,7 @@ const uint16_t tbl_for_zero_width_chars[][2] = {
      { 0x0D62, 0x0D63 }, { 0x0D81, 0x0D81 }, { 0x0DCA, 0x0DCA },
      { 0x0DD2, 0x0DD6 }, { 0x0E31, 0x0E31 }, { 0x0E34, 0x0E3A },
      { 0x0E47, 0x0E4E }, { 0x0EB1, 0x0EB1 }, { 0x0EB4, 0x0EBC },
-     { 0x0EC8, 0x0ECD }, { 0x0F18, 0x0F19 }, { 0x0F35, 0x0F35 },
+     { 0x0EC8, 0x0ECE }, { 0x0F18, 0x0F19 }, { 0x0F35, 0x0F35 },
      { 0x0F37, 0x0F37 }, { 0x0F39, 0x0F39 }, { 0x0F71, 0x0F7E },
      { 0x0F80, 0x0F84 }, { 0x0F86, 0x0F87 }, { 0x0F8D, 0x0FBC },
      { 0x0FC6, 0x0FC6 }, { 0x102D, 0x1030 }, { 0x1032, 0x1037 },
@@ -295,24 +295,27 @@ const uint16_t tbl_for_zero_width_chars[][2] = {
      { 0xFE00, 0xFE0F }, { 0xFE20, 0xFE2F }, { 0xFEFF, 0xFEFF },
      { 0xFFF9, 0xFFFB }, { 0x01FD, 0x01FD }, { 0x02E0, 0x02E0 },
      { 0x0376, 0x037A }, { 0x0A01, 0x0A0F }, { 0x0A38, 0x0A3F },
-     { 0x0AE5, 0x0AE6 }, { 0x0D24, 0x0D27 }, { 0x0EAB, 0x0EAC },
-     { 0x0F46, 0x0F50 }, { 0x0F82, 0x0F85 }, { 0x1001, 0x1001 },
-     { 0x1038, 0x1046 }, { 0x1070, 0x1070 }, { 0x1073, 0x1074 },
-     { 0x107F, 0x1081 }, { 0x10B3, 0x10B6 }, { 0x10B9, 0x10BA },
-     { 0x10BD, 0x10BD }, { 0x10C2, 0x10CD }, { 0x1100, 0x1102 },
-     { 0x1127, 0x112B }, { 0x112D, 0x1134 }, { 0x1173, 0x1173 },
-     { 0x1180, 0x1181 }, { 0x11B6, 0x11BE }, { 0x11C9, 0x11CC },
-     { 0x11CF, 0x11CF }, { 0x122F, 0x1231 }, { 0x1234, 0x1234 },
-     { 0x1236, 0x1237 }, { 0x123E, 0x123E }, { 0x12DF, 0x12DF },
+     { 0x0AE5, 0x0AE6 }, { 0x0D24, 0x0D27 }, { 0x0D69, 0x0D6D },
+     { 0x0EAB, 0x0EAC }, { 0x0EFC, 0x0EFF }, { 0x0F46, 0x0F50 },
+     { 0x0F82, 0x0F85 }, { 0x1001, 0x1001 }, { 0x1038, 0x1046 },
+     { 0x1070, 0x1070 }, { 0x1073, 0x1074 }, { 0x107F, 0x1081 },
+     { 0x10B3, 0x10B6 }, { 0x10B9, 0x10BA }, { 0x10BD, 0x10BD },
+     { 0x10C2, 0x10CD }, { 0x1100, 0x1102 }, { 0x1127, 0x112B },
+     { 0x112D, 0x1134 }, { 0x1173, 0x1173 }, { 0x1180, 0x1181 },
+     { 0x11B6, 0x11BE }, { 0x11C9, 0x11CC }, { 0x11CF, 0x11CF },
+     { 0x122F, 0x1231 }, { 0x1234, 0x1234 }, { 0x1236, 0x1237 },
+     { 0x123E, 0x123E }, { 0x1241, 0x1241 }, { 0x12DF, 0x12DF },
      { 0x12E3, 0x12EA }, { 0x1300, 0x1301 }, { 0x133B, 0x133C },
-     { 0x1340, 0x1340 }, { 0x1366, 0x1374 }, { 0x1438, 0x143F },
-     { 0x1442, 0x1444 }, { 0x1446, 0x1446 }, { 0x145E, 0x145E },
-     { 0x14B3, 0x14B8 }, { 0x14BA, 0x14BA }, { 0x14BF, 0x14C0 },
-     { 0x14C2, 0x14C3 }, { 0x15B2, 0x15B5 }, { 0x15BC, 0x15BD },
-     { 0x15BF, 0x15C0 }, { 0x15DC, 0x162F }, { 0x1633, 0x163A },
-     { 0x163D, 0x163D }, { 0x163F, 0x1640 }, { 0x16AB, 0x16AB },
-     { 0x16AD, 0x16AD }, { 0x16B0, 0x16B5 }, { 0x16B7, 0x16B7 },
-     { 0x171D, 0x171F }, { 0x1722, 0x1725 }, { 0x1727, 0x172B },
+     { 0x1340, 0x1340 }, { 0x1366, 0x1374 }, { 0x13BB, 0x13C0 },
+     { 0x13CE, 0x13CE }, { 0x13D0, 0x13D0 }, { 0x13D2, 0x13D2 },
+     { 0x13E1, 0x13E2 }, { 0x1438, 0x143F }, { 0x1442, 0x1444 },
+     { 0x1446, 0x1446 }, { 0x145E, 0x145E }, { 0x14B3, 0x14B8 },
+     { 0x14BA, 0x14BA }, { 0x14BF, 0x14C0 }, { 0x14C2, 0x14C3 },
+     { 0x15B2, 0x15B5 }, { 0x15BC, 0x15BD }, { 0x15BF, 0x15C0 },
+     { 0x15DC, 0x162F }, { 0x1633, 0x163A }, { 0x163D, 0x163D },
+     { 0x163F, 0x1640 }, { 0x16AB, 0x16AB }, { 0x16AD, 0x16AD },
+     { 0x16B0, 0x16B5 }, { 0x16B7, 0x16B7 }, { 0x171D, 0x171D },
+     { 0x171F, 0x171F }, { 0x1722, 0x1725 }, { 0x1727, 0x172B },
      { 0x182F, 0x1837 }, { 0x1839, 0x183A }, { 0x193B, 0x193C },
      { 0x193E, 0x193E }, { 0x1943, 0x1943 }, { 0x19D4, 0x19DB },
      { 0x19E0, 0x19E0 }, { 0x1A01, 0x1A0A }, { 0x1A33, 0x1A38 },
@@ -322,21 +325,25 @@ const uint16_t tbl_for_zero_width_chars[][2] = {
      { 0x1CAA, 0x1CB0 }, { 0x1CB2, 0x1CB3 }, { 0x1CB5, 0x1CB6 },
      { 0x1D31, 0x1D45 }, { 0x1D47, 0x1D47 }, { 0x1D90, 0x1D91 },
      { 0x1D95, 0x1D95 }, { 0x1D97, 0x1D97 }, { 0x1EF3, 0x1EF4 },
-     { 0x3430, 0x3438 }, { 0x6AF0, 0x6AF4 }, { 0x6B30, 0x6B36 },
-     { 0x6F4F, 0x6F4F }, { 0x6F8F, 0x6F92 }, { 0x6FE4, 0x6FE4 },
-     { 0xBC9D, 0xBC9E }, { 0xBCA0, 0xCF46 }, { 0xD167, 0xD169 },
+     { 0x1F00, 0x1F01 }, { 0x1F36, 0x1F3A }, { 0x1F40, 0x1F40 },
+     { 0x1F42, 0x1F42 }, { 0x1F5A, 0x1F5A }, { 0x3430, 0x3440 },
+     { 0x3447, 0x3455 }, { 0x611E, 0x6129 }, { 0x612D, 0x612F },
+     { 0x6AF0, 0x6AF4 }, { 0x6B30, 0x6B36 }, { 0x6F4F, 0x6F4F },
+     { 0x6F8F, 0x6F92 }, { 0x6FE4, 0x6FE4 }, { 0xBC9D, 0xBC9E },
+     { 0xBCA0, 0xBCA3 }, { 0xCF00, 0xCF46 }, { 0xD167, 0xD169 },
      { 0xD173, 0xD182 }, { 0xD185, 0xD18B }, { 0xD1AA, 0xD1AD },
      { 0xD242, 0xD244 }, { 0xDA00, 0xDA36 }, { 0xDA3B, 0xDA6C },
      { 0xDA75, 0xDA75 }, { 0xDA84, 0xDA84 }, { 0xDA9B, 0xDAAF },
-     { 0xE000, 0xE02A }, { 0xE130, 0xE136 }, { 0xE2AE, 0xE2AE },
-     { 0xE2EC, 0xE2EF }, { 0xE8D0, 0xE8D6 }, { 0xE944, 0xE94A },
+     { 0xE000, 0xE02A }, { 0xE08F, 0xE08F }, { 0xE130, 0xE136 },
+     { 0xE2AE, 0xE2AE }, { 0xE2EC, 0xE2EF }, { 0xE4EC, 0xE4EF },
+     { 0xE5EE, 0xE5EF }, { 0xE8D0, 0xE8D6 }, { 0xE944, 0xE94A },
      { 0x0001, 0x01EF } };
 
 const uint16_t tbl_for_fullwidth_chars[][2] = {
-     { /* plane 0 */ 0, 46 },
-     { /* plane 1 */ 46, 80 },
-     { /* plane 2 */ 80, 81 },
-     { /* plane 3 */ 81, 82 },
+     { /* plane 0 */ 0, 47 },
+     { /* plane 1 */ 47, 82 },
+     { /* plane 2 */ 82, 83 },
+     { /* plane 3 */ 83, 84 },
      { /* plane 4 */ 0, 0 },
      { /* plane 5 */ 0, 0 },
      { /* plane 6 */ 0, 0 },
@@ -352,32 +359,32 @@ const uint16_t tbl_for_fullwidth_chars[][2] = {
      { /* plane 16 */ 0, 0 },
      { 0x1100, 0x115F }, { 0x231A, 0x231B }, { 0x2329, 0x232A },
      { 0x23E9, 0x23EC }, { 0x23F0, 0x23F0 }, { 0x23F3, 0x23F3 },
-     { 0x25FD, 0x25FE }, { 0x2614, 0x2615 }, { 0x2648, 0x2653 },
-     { 0x267F, 0x267F }, { 0x2693, 0x2693 }, { 0x26A1, 0x26A1 },
-     { 0x26AA, 0x26AB }, { 0x26BD, 0x26BE }, { 0x26C4, 0x26C5 },
-     { 0x26CE, 0x26CE }, { 0x26D4, 0x26D4 }, { 0x26EA, 0x26EA },
-     { 0x26F2, 0x26F3 }, { 0x26F5, 0x26F5 }, { 0x26FA, 0x26FA },
-     { 0x26FD, 0x26FD }, { 0x2705, 0x2705 }, { 0x270A, 0x270B },
-     { 0x2728, 0x2728 }, { 0x274C, 0x274C }, { 0x274E, 0x274E },
-     { 0x2753, 0x2755 }, { 0x2757, 0x2757 }, { 0x2795, 0x2797 },
-     { 0x27B0, 0x27B0 }, { 0x27BF, 0x27BF }, { 0x2B1B, 0x2B1C },
-     { 0x2B50, 0x2B50 }, { 0x2B55, 0x2B55 }, { 0x2E80, 0x303E },
-     { 0x3041, 0x3247 }, { 0x3250, 0x4DBF }, { 0x4E00, 0xA4C6 },
-     { 0xA960, 0xA97C }, { 0xAC00, 0xD7A3 }, { 0xF900, 0xFAFF },
-     { 0xFE10, 0xFE19 }, { 0xFE30, 0xFE6B }, { 0xFF01, 0xFF60 },
-     { 0xFFE0, 0xFFE6 }, { 0x6FE0, 0xB2FB }, { 0xF004, 0xF004 },
-     { 0xF0CF, 0xF0CF }, { 0xF18E, 0xF18E }, { 0xF191, 0xF19A },
-     { 0xF200, 0xF320 }, { 0xF32D, 0xF335 }, { 0xF337, 0xF37C },
-     { 0xF37E, 0xF393 }, { 0xF3A0, 0xF3CA }, { 0xF3CF, 0xF3D3 },
-     { 0xF3E0, 0xF3F0 }, { 0xF3F4, 0xF3F4 }, { 0xF3F8, 0xF43E },
-     { 0xF440, 0xF440 }, { 0xF442, 0xF4FC }, { 0xF4FF, 0xF53D },
-     { 0xF54B, 0xF54E }, { 0xF550, 0xF567 }, { 0xF57A, 0xF57A },
-     { 0xF595, 0xF596 }, { 0xF5A4, 0xF5A4 }, { 0xF5FB, 0xF64F },
-     { 0xF680, 0xF6C5 }, { 0xF6CC, 0xF6CC }, { 0xF6D0, 0xF6D2 },
-     { 0xF6D5, 0xF6DF }, { 0xF6EB, 0xF6EC }, { 0xF6F4, 0xF6FC },
-     { 0xF7E0, 0xF7F0 }, { 0xF90C, 0xF93A }, { 0xF93C, 0xF945 },
-     { 0xF947, 0xF9FF }, { 0xFA70, 0xFAF6 }, { 0x0000, 0xFFFF },
-     { 0x0000, 0xFFFD } };
+     { 0x25FD, 0x25FE }, { 0x2614, 0x2615 }, { 0x2630, 0x2637 },
+     { 0x2648, 0x2653 }, { 0x267F, 0x267F }, { 0x268A, 0x268F },
+     { 0x2693, 0x2693 }, { 0x26A1, 0x26A1 }, { 0x26AA, 0x26AB },
+     { 0x26BD, 0x26BE }, { 0x26C4, 0x26C5 }, { 0x26CE, 0x26CE },
+     { 0x26D4, 0x26D4 }, { 0x26EA, 0x26EA }, { 0x26F2, 0x26F3 },
+     { 0x26F5, 0x26F5 }, { 0x26FA, 0x26FA }, { 0x26FD, 0x26FD },
+     { 0x2705, 0x2705 }, { 0x270A, 0x270B }, { 0x2728, 0x2728 },
+     { 0x274C, 0x274C }, { 0x274E, 0x274E }, { 0x2753, 0x2755 },
+     { 0x2757, 0x2757 }, { 0x2795, 0x2797 }, { 0x27B0, 0x27B0 },
+     { 0x27BF, 0x27BF }, { 0x2B1B, 0x2B1C }, { 0x2B50, 0x2B50 },
+     { 0x2B55, 0x2B55 }, { 0x2E80, 0x303E }, { 0x3041, 0x3247 },
+     { 0x3250, 0xA4C6 }, { 0xA960, 0xA97C }, { 0xAC00, 0xD7A3 },
+     { 0xF900, 0xFAFF }, { 0xFE10, 0xFE19 }, { 0xFE30, 0xFE6B },
+     { 0xFF01, 0xFF60 }, { 0xFFE0, 0xFFE6 }, { 0x6FE0, 0xB2FB },
+     { 0xD300, 0xD376 }, { 0xF004, 0xF004 }, { 0xF0CF, 0xF0CF },
+     { 0xF18E, 0xF18E }, { 0xF191, 0xF19A }, { 0xF200, 0xF320 },
+     { 0xF32D, 0xF335 }, { 0xF337, 0xF37C }, { 0xF37E, 0xF393 },
+     { 0xF3A0, 0xF3CA }, { 0xF3CF, 0xF3D3 }, { 0xF3E0, 0xF3F0 },
+     { 0xF3F4, 0xF3F4 }, { 0xF3F8, 0xF43E }, { 0xF440, 0xF440 },
+     { 0xF442, 0xF4FC }, { 0xF4FF, 0xF53D }, { 0xF54B, 0xF54E },
+     { 0xF550, 0xF567 }, { 0xF57A, 0xF57A }, { 0xF595, 0xF596 },
+     { 0xF5A4, 0xF5A4 }, { 0xF5FB, 0xF64F }, { 0xF680, 0xF6C5 },
+     { 0xF6CC, 0xF6CC }, { 0xF6D0, 0xF6D2 }, { 0xF6D5, 0xF6DF },
+     { 0xF6EB, 0xF6EC }, { 0xF6F4, 0xF6FC }, { 0xF7E0, 0xF7F0 },
+     { 0xF90C, 0xF93A }, { 0xF93C, 0xF945 }, { 0xF947, 0xF9FF },
+     { 0xFA70, 0xFAF8 }, { 0x0000, 0xFFFF }, { 0x0000, 0xFFFD } };
 
   /* test for 8-bit control characters */
     if (ucs == 0)
@@ -405,7 +412,7 @@ const uint16_t tbl_for_fullwidth_chars[][2] = {
 
 /* The handling of "fullwidth" characters (those consuming two "normal"
 columns) and combining characters (characters that can add accents to a
-preceding character) in PDCurses is,  of necessity,  complex.
+preceding character) in PDCursesMod is,  of necessity,  complex.
 
 Unicode is defined to have 17 planes of 2^16 characters each,  so that
 the maximum Unicode code point is U+10FFFF.  When addch() is given a
@@ -414,7 +421,7 @@ stores the non-Unicode character DUMMY_CHAR_NEXT_TO_FULLWIDTH
 (U+110000) next to it,  just as a placeholder.  (That part is actually
 rather simple.)
 
-PDCurses handles combining characters by creating entirely new "Unicode"
+PDCursesMod handles combining characters by creating entirely new "Unicode"
 (let's call them "post-Unicode") characters,  at code point U+110001
 (COMBINED_CHAR_START) and beyond. The 'combos' table keeps track of
 these newly-created characters,  essentially saying:  "This post-Unicode
@@ -439,7 +446,7 @@ actual series of characters.
 'ncurses' handles combined characters in a very different manner:  a
 'cchar' is defined as an array of five characters,  so that you can
 add up to four combining characters in any given cell.  I had to reject
-that solution because backward compatibility within PDCurses would be
+that solution because backward compatibility within PDCurseMod would be
 broken.  Quite aside from that,  this is a simpler solution,  and allows
 for any number of combining characters (though four ought to be enough
 for anybody).      */
