@@ -1,4 +1,4 @@
-/* PDCurses */
+/* PDCursesMod */
 
 #include <curspriv.h>
 
@@ -224,8 +224,6 @@ static void _check_hash_tbl( void)
       }
 }
 
-static int _default_foreground_idx = COLOR_WHITE, _default_background_idx = COLOR_BLACK;
-
 int start_color(void)
 {
     PDC_LOG(("start_color() - called\n"));
@@ -241,8 +239,8 @@ int start_color(void)
     if (!SP->default_colors && SP->orig_attr && getenv("PDC_ORIGINAL_COLORS"))
         SP->default_colors = TRUE;
 
-    _init_pair_core( 0, _default_foreground_idx,
-                        _default_background_idx);
+    _init_pair_core( 0, SP->default_foreground_idx,
+                        SP->default_background_idx);
     if( !SP->_preserve)
         curscr->_clear = TRUE;
 #if !defined( CHTYPE_32) && !defined(OS2) && !defined(DOS)
@@ -263,8 +261,8 @@ int start_color(void)
 
 void PDC_set_default_colors( const int fg_idx, const int bg_idx)
 {
-   _default_foreground_idx = fg_idx;
-   _default_background_idx = bg_idx;
+   SP->default_foreground_idx = fg_idx;
+   SP->default_background_idx = bg_idx;
 }
 
 static void _normalize(int *fg, int *bg)
@@ -272,10 +270,10 @@ static void _normalize(int *fg, int *bg)
     const bool using_defaults = (SP->orig_attr && (SP->default_colors || !SP->color_started));
 
     if (*fg == -1 || *fg == UNSET_COLOR_PAIR)
-        *fg = using_defaults ? SP->orig_fore : _default_foreground_idx;
+        *fg = using_defaults ? SP->orig_fore : SP->default_foreground_idx;
 
     if (*bg == -1 || *fg == UNSET_COLOR_PAIR)
-        *bg = using_defaults ? SP->orig_back : _default_background_idx;
+        *bg = using_defaults ? SP->orig_back : SP->default_background_idx;
 }
 
 /* When a color pair is reset,  all cells of that color should be
@@ -316,7 +314,7 @@ PDC_set_line_color() is called (and changes the way in which text
 with those attributes is drawn),  the corresponding text should be
 redrawn.    */
 
-void PDC_set_cells_to_refresh_for_attr_change( const attr_t attr)
+static void _set_cells_to_refresh_for_attr_change( const chtype attr)
 {
     int x, y;
 
@@ -534,8 +532,8 @@ int PDC_set_line_color(short color)
     if( SP->line_color != color)
     {
         SP->line_color = color;
-        PDC_set_cells_to_refresh_for_attr_change(
-               WA_TOP | WA_UNDERLINE | WA_LEFT | WA_RIGHT | WA_STRIKEOUT);
+        _set_cells_to_refresh_for_attr_change(
+               A_TOP | A_UNDERLINE | A_LEFT | A_RIGHT | A_STRIKEOUT);
     }
     return OK;
 }
@@ -554,7 +552,7 @@ static void _init_color_table( SCREEN *sp)
     p[0].prev = p[0].next = 0;
     p[1].prev = p[1].next = 1;
     sp->default_colors = FALSE;
-    PDC_set_default_colors( _default_foreground_idx, _default_background_idx);
+    PDC_set_default_colors( COLOR_WHITE, COLOR_BLACK);
 }
 
 int PDC_init_atrtab(void)
@@ -562,8 +560,8 @@ int PDC_init_atrtab(void)
     assert( SP);
     _init_color_table( SP);
     _init_pair_core( 0,
-            (SP->orig_attr ? SP->orig_fore : _default_foreground_idx),
-            (SP->orig_attr ? SP->orig_back : _default_background_idx));
+            (SP->orig_attr ? SP->orig_fore : SP->default_foreground_idx),
+            (SP->orig_attr ? SP->orig_back : SP->default_background_idx));
     return( 0);
 }
 
@@ -697,8 +695,8 @@ void reset_color_pairs( void)
     SP->pair_hash_tbl_size = SP->pair_hash_tbl_used = 0;
     _init_color_table( SP);
     _init_pair_core( 0,
-            (SP->orig_attr ? SP->orig_fore : _default_foreground_idx),
-            (SP->orig_attr ? SP->orig_back : _default_background_idx));
+            (SP->orig_attr ? SP->orig_fore : SP->default_foreground_idx),
+            (SP->orig_attr ? SP->orig_back : SP->default_background_idx));
     curscr->_clear = TRUE;
 }
 
