@@ -1,4 +1,4 @@
-/* PDCurses */
+/* PDCursesMod */
 
 #include <stdlib.h>
 #include <curspriv.h>
@@ -203,6 +203,8 @@ WINDOW *PDC_makelines(WINDOW *win)
     assert( win);
     if (!win)
         return (WINDOW *)NULL;
+    assert( win->_maxy > 0);
+    assert( win->_maxx > 0);
 
     nlines = win->_maxy;
     ncols = win->_maxx;
@@ -237,7 +239,7 @@ exists for all platforms,  and always frees the pointer and returns NULL for a
 zero-byte allocation.  realloc() does this on most platforms, but not FreeBSD,
 and it's not guaranteed in the C specifications. */
 
-static void *PDC_realloc_array( void *ptr, const size_t nmemb, const size_t size)
+void *PDC_realloc_array( void *ptr, const size_t nmemb, const size_t size)
 {
     if( !nmemb || !size)
     {
@@ -289,9 +291,19 @@ WINDOW *newwin(int nlines, int ncols, int begy, int begx)
         return (WINDOW *)NULL;
 
     assert( SP);
-    assert( begy + nlines <= SP->lines && begx + ncols <= SP->cols);
-    if (!SP || begy + nlines > SP->lines || begx + ncols > SP->cols)
+    if( !SP)
         return (WINDOW *)NULL;
+    if( !(SP->off_screen_windows & OFF_SCREEN_WINDOWS_TO_RIGHT_AND_BOTTOM))
+    {
+        if( begy + nlines > SP->lines || begx + ncols > SP->cols)
+            return (WINDOW *)NULL;
+    }
+
+    if( !(SP->off_screen_windows & OFF_SCREEN_WINDOWS_TO_LEFT_AND_TOP))
+    {
+    if( begy < 0 || begx < 0)
+        return (WINDOW *)NULL;
+    }
 
     win = PDC_makenew(nlines, ncols, begy, begx);
     if (win)
@@ -355,11 +367,19 @@ int mvwin(WINDOW *win, int y, int x)
     PDC_LOG(("mvwin() - called\n"));
 
     assert( win);
-    assert( y + win->_maxy <= LINES && y >= 0
-         && x + win->_maxx <= COLS && x >=0);
-    if (!win || (y + win->_maxy > LINES || y < 0)
-             || (x + win->_maxx > COLS || x < 0))
+    if( !win)
         return ERR;
+    if( !(SP->off_screen_windows & OFF_SCREEN_WINDOWS_TO_LEFT_AND_TOP))
+    {
+        if( y < 0 || x < 0)
+            return ERR;
+    }
+
+    if( !(SP->off_screen_windows & OFF_SCREEN_WINDOWS_TO_RIGHT_AND_BOTTOM))
+    {
+        if( y + win->_maxy > LINES || x + win->_maxx > COLS)
+            return ERR;
+    }
 
     win->_begy = y;
     win->_begx = x;
