@@ -292,6 +292,7 @@ static int _process_key_event(void)
                 (key_table[i].numkeypad && (event.key.keysym.mod & KMOD_NUM)))
             {
                 key = key_table[i].shifted;
+/* Is the following really Linux-only? */
                 if( (key >= '0' && key <= '9') || strchr( ".+-*/", key))
                     key = -1;
             }
@@ -355,7 +356,7 @@ static int _process_mouse_event(void)
 
     SP->mouse_status.x = (event.motion.x - pdc_xoffset) / pdc_fwidth;
     SP->mouse_status.y = (event.motion.y - pdc_yoffset) / pdc_fheight;
-    if( SP->mouse_status.x >= COLS || SP->mouse_status.y >= LINES)
+    if( SP->mouse_status.x >= COLS || SP->mouse_status.y >= SP->lines)
         return -1;
 
     if (event.type == SDL_MOUSEMOTION)
