@@ -135,7 +135,7 @@ box) in place of any visible cursor.  */
 static int PDC_current_cursor_state( void)
 {
     extern HWND PDC_hWnd;
-    const int shift_amount = (PDC_blink_state ? 0 : 8);
+    const int shift_amount = (SP->blink_state ? 0 : 8);
     const int cursor_style_for_unfocussed_window =
                PDC_CURSOR( PDC_CURSOR_OUTLINE, PDC_CURSOR_OUTLINE);
     int cursor_style;
@@ -392,26 +392,10 @@ static void PDC_transform_line_given_hdc( const HDC hdc, const int lineno,
 #endif
         return;
     }
-                     /* Seems to me as if the input text to this function */
-    if( x < 0)       /* should _never_ be off-screen.  But it sometimes is. */
-    {                /* Clipping is therefore necessary. */
-        len += x;
-        srcp -= x;
-        x = 0;
-    }
-    if( (srcp[len] & A_CHARTEXT) < MAX_UNICODE)
-       len++;    /* draw an extra char to avoid leaving garbage on screen */
-    if( len > SP->cols - x)
-        len = SP->cols - x;
-    if( lineno >= SP->lines || len <= 0 || lineno < 0)
-        return;
+    assert( x >= 0);
+    assert( len <= SP->cols - x);
+    assert( lineno < SP->lines && len > 0 && lineno >= 0);
     assert( (srcp[len - 1] & A_CHARTEXT) != MAX_UNICODE);
-    if( x && (srcp[-1] & A_CHARTEXT) < MAX_UNICODE)
-    {                /* back up by one character to avoid */
-        x--;         /* leaving garbage on the screen */
-        len++;
-        srcp--;
-    }
     if( lineno == SP->cursrow && SP->curscol >= x && SP->curscol < x + len)
         if( PDC_current_cursor_state( ))
             cursor_overwritten = TRUE;
@@ -423,6 +407,7 @@ static void PDC_transform_line_given_hdc( const HDC hdc, const int lineno,
         attr_t new_font_attrib = (*srcp & (A_BOLD | A_ITALIC));
         RECT clip_rect;
         wchar_t buff[BUFFSIZE];
+        int lpDx[BUFFSIZE + 1];
         int olen = 0;
 #ifdef USE_FALLBACK_FONT
         const bool in_font = character_is_in_font( *srcp);
@@ -443,6 +428,7 @@ static void PDC_transform_line_given_hdc( const HDC hdc, const int lineno,
             {               /* >64K values into 16-bit wchar_t: */
                 ch -= 0x10000;
                 buff[olen] = (wchar_t)( 0xd800 | (ch >> 10));
+                lpDx[olen] = 0;
                 olen++;                           /* ^ upper 10 bits */
                 ch = (wchar_t)( 0xdc00 | (ch & 0x3ff));  /* lower 10 bits */
             }
@@ -457,12 +443,14 @@ static void PDC_transform_line_given_hdc( const HDC hdc, const int lineno,
                     n_combined++;
                 }
                 buff[olen] = (wchar_t)root;
+                lpDx[olen] = 0;
                 olen++;
                 ch = (wchar_t)added[n_combined];
                 while( n_combined)
                 {
                     n_combined--;
                     buff[olen] = (wchar_t)added[n_combined];
+                    lpDx[olen] = 0;
                     olen++;
                 }
             }
@@ -482,8 +470,17 @@ static void PDC_transform_line_given_hdc( const HDC hdc, const int lineno,
             }
 #endif
             buff[olen] = (wchar_t)ch;
+            lpDx[olen] = PDC_cxChar;
+#ifdef PDC_WIDE
+            if( ch != MAX_UNICODE)
+                olen++;
+            else if( olen)          /* prev char is double-width */
+                lpDx[olen - 1] = 2 * PDC_cxChar;
+#else
             olen++;
+#endif
         }
+        lpDx[olen] = PDC_cxChar;
         if( color != curr_color || ((prev_ch ^ *srcp) & (A_REVERSE | A_BLINK | A_BOLD | A_DIM)))
         {
             PACKED_RGB background_rgb;
@@ -528,7 +525,7 @@ static void PDC_transform_line_given_hdc( const HDC hdc, const int lineno,
         clip_rect.bottom = clip_rect.top + PDC_cyChar;
         ExtTextOutW( hdc, clip_rect.left, clip_rect.top,
                            ETO_CLIPPED | ETO_OPAQUE, &clip_rect,
-                           buff, olen, NULL);
+                           buff, olen, (olen > 1 ? lpDx : NULL));
 #ifdef WA_TOP
         if( *srcp & (WA_UNDERLINE | WA_RIGHT | WA_LEFT | WA_TOP | WA_STRIKEOUT))
 #else
@@ -607,4 +604,11 @@ void PDC_transform_line(int lineno, int x, int len, const chtype *srcp)
 
 void PDC_doupdate(void)
 {
+    MSG msg;
+
+    while( PeekMessage(&msg, NULL, 0, 0, PM_REMOVE) )
+    {
+       TranslateMessage(&msg);
+       DispatchMessage(&msg);
+    }
 }
