@@ -4,6 +4,7 @@
 #include <tchar.h>
 #include <assert.h>
 #include "../common/pdccolor.h"
+#include "../common/mouse.c"
 #ifdef WIN32_LEAN_AND_MEAN
    #ifdef PDC_WIDE
       #include <shellapi.h>
@@ -14,8 +15,8 @@
 /* COLOR_PAIR to attribute encoding table. */
 
 static int menu_shown = 1;
-static int min_lines = 25, max_lines = 25;
-static int min_cols = 80, max_cols = 80;
+static int min_lines = 20, max_lines = 200;
+static int min_cols = 60, max_cols = 800;
 
 #if !defined( CHTYPE_32) && defined( PDC_WIDE)
     #define USING_COMBINING_CHARACTER_SCHEME
@@ -33,7 +34,6 @@ functions.        */
 #define INLINE static inline
 #endif
 
-static int add_mouse( int button, const int action, const int x, const int y);
 static int keep_size_within_bounds( int *lines, int *cols);
 INLINE int set_default_sizes_from_registry( const int n_cols, const int n_rows,
                const int xloc, const int yloc);
@@ -45,9 +45,9 @@ cube of 16 million colors. */
 #define N_COLORS 256 + 256 * 256 * 256;
 
 #ifdef WA_TOP
-#define WA_ALL_LINES (WA_UNDERLINE | WA_LOW | WA_LEFT | WA_RIGHT | WA_TOP | WA_STRIKEOUT)
+#define WA_ALL_LINES (WA_UNDERLINE | WA_LEFT | WA_RIGHT | WA_TOP | WA_STRIKEOUT)
 #else
-#define WA_ALL_LINES (WA_UNDERLINE | WA_LOW | WA_LEFT | WA_RIGHT)
+#define WA_ALL_LINES (WA_UNDERLINE | WA_LEFT | WA_RIGHT)
 #endif
 
    /* If PDC_MAX_MOUSE_BUTTONS is undefined,  it means the user hasn't     */
@@ -1488,12 +1488,25 @@ static void HandleSyskeyDown( const WPARAM wParam, const LPARAM lParam,
         SP->key_modifiers |= PDC_KEY_MODIFIER_REPEAT;
 }
 
+static int convert_key_modifiers_for_mouse( const int key_modifiers)
+{
+   int mouse_modifiers = 0;
+
+   if( key_modifiers & PDC_KEY_MODIFIER_ALT)
+      mouse_modifiers |= PDC_BUTTON_ALT;
+   if( key_modifiers & PDC_KEY_MODIFIER_SHIFT)
+      mouse_modifiers |= PDC_BUTTON_SHIFT;
+   if( key_modifiers & PDC_KEY_MODIFIER_CONTROL)
+      mouse_modifiers |= PDC_BUTTON_CONTROL;
+   return( mouse_modifiers);
+}
+
 static void HandleTimer( const WPARAM wParam )
 {
     int i;           /* see WndProc() notes */
 
     INTENTIONALLY_UNUSED_PARAMETER( wParam);
-    PDC_blink_state ^= 1;
+    SP->blink_state ^= 1;
     if( SP->termattrs & A_BLINK)
     {
         for( i = 0; i < SP->lines; i++)
@@ -1579,163 +1592,12 @@ INLINE void HandleMenuToggle( bool *ptr_ignore_resize)
     InvalidateRect( PDC_hWnd, NULL, FALSE);
 }
 
-typedef struct
-{
-   int x, y;
-   int button, action;
-   int button_flags;    /* Alt, shift, ctrl */
-} PDC_mouse_event;
-
-/* As "combined" mouse events (i.e.,  clicks and double- and triple-clicks
-along with the usual mouse moves,  button presses and releases,  and wheel
-movements) occur,  we add them to a queue.  They are removed for each
-KEY_MOUSE event from getch( ),  and SP->mouse_status is set to reflect
-what the mouse was doing at that event.
-
-Seven queued mouse events is possibly overkill.       */
-
-#define MAX_MOUSE_QUEUE       7
-
-static PDC_mouse_event mouse_queue[MAX_MOUSE_QUEUE];
-static int n_mouse_queue = 0;
-
-int PDC_get_mouse_event_from_queue( void)
-{
-    size_t i;
-
-    if( !n_mouse_queue)
-        return( -1);
-    memset(&SP->mouse_status, 0, sizeof(MOUSE_STATUS));
-    if( mouse_queue->action == BUTTON_MOVED)
-    {
-        if( mouse_queue->button < 0)
-            SP->mouse_status.changes = PDC_MOUSE_MOVED;
-        else
-        {
-            SP->mouse_status.changes = PDC_MOUSE_MOVED | (1 << mouse_queue->button);
-            SP->mouse_status.button[mouse_queue->button] = BUTTON_MOVED;
-        }
-    }
-    else
-    {
-        if( mouse_queue->button < PDC_MAX_MOUSE_BUTTONS)
-        {
-            SP->mouse_status.button[mouse_queue->button] = (short)mouse_queue->action;
-            if( mouse_queue->button < 3)
-               SP->mouse_status.changes = (1 << mouse_queue->button);
-            else
-               SP->mouse_status.changes = (0x40 << mouse_queue->button);
-        }
-        else if( mouse_queue->button == WHEEL_EVENT)
-             SP->mouse_status.changes |= mouse_queue->action;
-    }
-    SP->mouse_status.x = mouse_queue->x;
-    SP->mouse_status.y = mouse_queue->y;
-    for (i = 0; i < PDC_MAX_MOUSE_BUTTONS; i++)
-        SP->mouse_status.button[i] |= mouse_queue->button_flags;
-    n_mouse_queue--;
-    memmove( mouse_queue, mouse_queue + 1, n_mouse_queue * sizeof( PDC_mouse_event));
-    return( 0);
-}
-
-static void add_mouse_event_to_queue( const int button, const int action,
-            const int x, const int y)
-{
-    if( x < PDC_n_cols && y < PDC_n_rows && n_mouse_queue < MAX_MOUSE_QUEUE)
-    {
-        int button_flags = 0;
-
-        mouse_queue[n_mouse_queue].button = button;
-        mouse_queue[n_mouse_queue].action = action;
-        mouse_queue[n_mouse_queue].x = x;
-        mouse_queue[n_mouse_queue].y = y;
-        if( GetKeyState( VK_MENU) & 0x8000)
-            button_flags |= PDC_BUTTON_ALT;
-
-        if( GetKeyState( VK_SHIFT) & 0x8000)
-            button_flags |= PDC_BUTTON_SHIFT;
-
-        if( GetKeyState( VK_CONTROL) & 0x8000)
-            button_flags |= PDC_BUTTON_CONTROL;
-        mouse_queue[n_mouse_queue].button_flags = button_flags;
-        n_mouse_queue++;
-        add_key_to_queue( KEY_MOUSE);
-    }
-}
-
-/* 'button_count' is zero if a button hasn't been pressed;  one if it
-has been;  two if pressed/released (clicked);  three if clicked and
-pressed again... all the way up to six if it's been triple-clicked. */
-
-static int add_mouse( int button, const int action, const int x, const int y)
-{
-   bool flush_events_to_queue = (button == -1 || action == BUTTON_MOVED);
-   static int mouse_state = 0, button_count[PDC_MAX_MOUSE_BUTTONS];
-   static int prev_x, prev_y = -1;
-   const bool actually_moved = (x != prev_x || y != prev_y);
-   size_t i;
-
-   if( action == BUTTON_RELEASED)
-   {
-       mouse_state &= ~(1 << button);
-       if( !button_count[button - 1])   /* a release with no matching press */
-       {
-           add_mouse_event_to_queue( button - 1, BUTTON_RELEASED, x, y);
-           return( 0);
-       }
-       else if( button_count[button - 1] & 1)
-       {
-           button_count[button - 1]++;
-           if( button_count[button - 1] == 6)    /* triple-click completed */
-               flush_events_to_queue = TRUE;
-       }
-   }
-   else if( action == BUTTON_PRESSED && !(button_count[button - 1] & 1))
-   {
-      mouse_state |= (1 << button);
-      button_count[button - 1]++;
-   }
-   if( button >= 0)
-   {
-      prev_x = x;
-      prev_y = y;
-   }
-   if( action == BUTTON_MOVED)
-   {
-       if( !actually_moved)     /* have to move to a new character cell, */
-           return( -1);         /* not just a new pixel */
-       button = -1;        /* assume no buttons down */
-       for( i = 0; i < PDC_MAX_MOUSE_BUTTONS; i++)
-           if( (mouse_state >> i) & 1)
-               button = (int)i;
-       if( button == -1 && !(SP->_trap_mbe & REPORT_MOUSE_POSITION))
-           return( -1);
-   }
-
-   if( flush_events_to_queue)
-       for( i = 0; i < PDC_MAX_MOUSE_BUTTONS; i++)
-           if( button_count[i])
-           {
-               const int events[4] = { 0, BUTTON_CLICKED,
-                           BUTTON_DOUBLE_CLICKED, BUTTON_TRIPLE_CLICKED };
-
-               assert( button_count[i] > 0 && button_count[i] < 7);
-               if( button_count[i] >= 2)
-                  add_mouse_event_to_queue( (int)i, events[button_count[i] / 2], prev_x, prev_y);
-               if( button_count[i] & 1)
-                  add_mouse_event_to_queue( (int)i, BUTTON_PRESSED, prev_x, prev_y);
-               button_count[i] = 0;
-           }
-   if( action == BUTTON_MOVED)
-      add_mouse_event_to_queue( button - 1, action, x, y);
-   debug_printf( "Button %d, act %d\n", button, action);
-   return( 0);
-}
-
 /* Note that there are two types of WM_TIMER timer messages.  One type
 indicates that SP->mouse_wait milliseconds have elapsed since a mouse
-button was pressed;  that's handled as described in the above notes.
-The other type,  issued every half second,  indicates that blinking
+button was pressed or released,  and that accumulated mouse events
+should be handled.  See '../common/mouse.c' for details.
+
+   The other type,  issued every half second,  indicates that blinking
 should take place.  For these,  HandleTimer() is called (see above).
 
    On WM_PAINT,  we determine what parts of 'curscr' would be covered by
@@ -1765,7 +1627,12 @@ static LRESULT ALIGN_STACK CALLBACK WndProc (const HWND hwnd,
     static int xbutton_pressed = 0;
     static int modified_key_to_return = 0;
     static bool ignore_resize = FALSE;
-    int button, action;
+    int button, i;
+    bool wait_for_more_mouse = FALSE;
+    const int modifiers = convert_key_modifiers_for_mouse( SP->key_modifiers);
+    const int mouse_x = LOWORD( lParam) / PDC_cxChar;
+    const int mouse_y = HIWORD( lParam) / PDC_cyChar;
+
 
     PDC_hWnd = hwnd;
     if( !hwnd)
@@ -1805,12 +1672,14 @@ static LRESULT ALIGN_STACK CALLBACK WndProc (const HWND hwnd,
                 while( mouse_wheel_vertical_loc > mouse_wheel_sensitivity / 2)
                 {
                     mouse_wheel_vertical_loc -= mouse_wheel_sensitivity;
-                    add_mouse_event_to_queue( WHEEL_EVENT, PDC_MOUSE_WHEEL_UP, pt.x, pt.y);
+                    _add_raw_mouse_event( 0, PDC_MOUSE_WHEEL_UP, modifiers,
+                                                mouse_x, mouse_y);
                 }
                 while( mouse_wheel_vertical_loc < -mouse_wheel_sensitivity / 2)
                 {
                     mouse_wheel_vertical_loc += mouse_wheel_sensitivity;
-                    add_mouse_event_to_queue( WHEEL_EVENT, PDC_MOUSE_WHEEL_DOWN, pt.x, pt.y);
+                    _add_raw_mouse_event( 0, PDC_MOUSE_WHEEL_DOWN, modifiers,
+                                                mouse_x, mouse_y);
                 }
             }
             else       /* must be a horizontal event: */
@@ -1819,60 +1688,63 @@ static LRESULT ALIGN_STACK CALLBACK WndProc (const HWND hwnd,
                 while( mouse_wheel_horizontal_loc > mouse_wheel_sensitivity / 2)
                 {
                     mouse_wheel_horizontal_loc -= mouse_wheel_sensitivity;
-                    add_mouse_event_to_queue( WHEEL_EVENT, PDC_MOUSE_WHEEL_RIGHT, pt.x, pt.y);
+                    _add_raw_mouse_event( 0, PDC_MOUSE_WHEEL_RIGHT, modifiers,
+                                                mouse_x, mouse_y);
                 }
                 while( mouse_wheel_horizontal_loc < -mouse_wheel_sensitivity / 2)
                 {
                     mouse_wheel_horizontal_loc += mouse_wheel_sensitivity;
-                    add_mouse_event_to_queue( WHEEL_EVENT, PDC_MOUSE_WHEEL_LEFT, pt.x, pt.y);
+                    _add_raw_mouse_event( 0, PDC_MOUSE_WHEEL_LEFT, modifiers,
+                                                mouse_x, mouse_y);
                 }
             }
         }
+        for( i = _mlist_count; i; i--)
+            add_key_to_queue( KEY_MOUSE);
         return 0;
 
     case WM_MOUSEMOVE:
         {
-        const int mouse_x = LOWORD( lParam) / PDC_cxChar;
-        const int mouse_y = HIWORD( lParam) / PDC_cyChar;
-
-        if( add_mouse( 0, BUTTON_MOVED, mouse_x, mouse_y))
-            modified_key_to_return = 0;
+        _add_raw_mouse_event( 0, BUTTON_MOVED, modifiers, mouse_x, mouse_y);
+        for( i = _mlist_count; i; i--)
+            add_key_to_queue( KEY_MOUSE);
         }
         return 0;
 
     case WM_LBUTTONDOWN:
-        button = 1;
-        action = BUTTON_PRESSED;
+        wait_for_more_mouse = _add_raw_mouse_event( 0, BUTTON_PRESSED,
+                                               modifiers, mouse_x, mouse_y);
         break;
 
     case WM_LBUTTONUP:
-        button = 1;
-        action = BUTTON_RELEASED;
+        wait_for_more_mouse = _add_raw_mouse_event( 0, BUTTON_RELEASED,
+                                               modifiers, mouse_x, mouse_y);
         break;
 
     case WM_RBUTTONDOWN:
-        button = 3;
-        action = BUTTON_PRESSED;
+        wait_for_more_mouse = _add_raw_mouse_event( 2, BUTTON_PRESSED,
+                                               modifiers, mouse_x, mouse_y);
         break;
 
     case WM_RBUTTONUP:
-        button = 3;
-        action = BUTTON_RELEASED;
+        wait_for_more_mouse = _add_raw_mouse_event( 2, BUTTON_RELEASED,
+                                               modifiers, mouse_x, mouse_y);
         break;
 
     case WM_MBUTTONDOWN:
-        button = 2;
-        action = BUTTON_PRESSED;
+        wait_for_more_mouse = _add_raw_mouse_event( 1, BUTTON_PRESSED,
+                                               modifiers, mouse_x, mouse_y);
         break;
 
     case WM_MBUTTONUP:
-        button = 2;
-        action = BUTTON_RELEASED;
+        wait_for_more_mouse = _add_raw_mouse_event( 1, BUTTON_RELEASED,
+                                               modifiers, mouse_x, mouse_y);
         break;
 
     case WM_XBUTTONDOWN:
         button = ((wParam & MK_XBUTTON1) ? 3 : 4);
-        action = BUTTON_PRESSED;
+        wait_for_more_mouse = _add_raw_mouse_event( button, BUTTON_PRESSED,
+                                               modifiers, mouse_x, mouse_y);
         xbutton_pressed = button;
         break;
 
@@ -1885,8 +1757,8 @@ static LRESULT ALIGN_STACK CALLBACK WndProc (const HWND hwnd,
           /* track of which buttons are currently pressed.                */
         button = ((wParam & MK_XBUTTON1) ? 3 : 4);
 #endif
-        button = xbutton_pressed;
-        action = BUTTON_RELEASED;
+        wait_for_more_mouse = _add_raw_mouse_event( xbutton_pressed,  BUTTON_RELEASED,
+                                               modifiers, mouse_x, mouse_y);
         break;
 
     case WM_MOVE:
@@ -1943,7 +1815,8 @@ static LRESULT ALIGN_STACK CALLBACK WndProc (const HWND hwnd,
         if( wParam != TIMER_ID_FOR_BLINKING)
         {
             KillTimer( PDC_hWnd, (int)wParam);
-            add_mouse( -1, -1, -1, -1);
+            for( i = _mlist_count; i; i--)
+               add_key_to_queue( KEY_MOUSE);
         }
         else if( SP && curscr && curscr->_y)
         {
@@ -1999,14 +1872,17 @@ static LRESULT ALIGN_STACK CALLBACK WndProc (const HWND hwnd,
         return DefWindowProc( hwnd, message, wParam, lParam) ;
     }
 
-    /* mouse button handling code */
-    assert(button != -1);
-    add_mouse(button, action, LOWORD(lParam) / PDC_cxChar, HIWORD(lParam) / PDC_cyChar);
+    if( wait_for_more_mouse)
+        SetTimer(hwnd, 0, SP->mouse_wait, NULL);
+    else       /* flush any cached mouse events */
+        for( i = _mlist_count; i; i--)
+            add_key_to_queue( KEY_MOUSE);
+#ifdef PROBABLY_NOT_NEEDED_NOW
     if (action == BUTTON_PRESSED)
         SetCapture(hwnd);
     else
         ReleaseCapture();
-    SetTimer(hwnd, 0, SP->mouse_wait, NULL);
+#endif
     return 0;
 }
 
